
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.rjconsultores.ventaboletos.entidad.Aidf;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioDocumentosFiscaisBean;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioDocumentosFiscaisAgrupadoPorLocalidade extends Relatorio {

	public RelatorioDocumentosFiscaisAgrupadoPorLocalidade(Map<String, Object> parametros, Connection conexao) {
		super(parametros, conexao);
	}

	private List<RelatorioDocumentosFiscaisBean> lsDadosRelatorio;

	@Override
	protected void processaParametros() throws Exception {

		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();

				Map<String, Object> parametros = this.relatorio.getParametros();

				Integer empresaId = (Integer) parametros.get("EMPRESA_ID");
				String agencia = (String) parametros.get("PUNTOVENTA");
				String tipoLinha = (String) parametros.get("TIPO_LINHA");
				Aidf aidf = (Aidf) parametros.get("AIDF");
				Boolean somenteCancelado = (Boolean) parametros.get("SOMENTE_CANCELADO");

				String sql = getSql(empresaId, agencia, somenteCancelado, tipoLinha, aidf);

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);

				stmt.setTimestamp("DATA_INICIAL", (Timestamp) parametros.get("DATA_INICIAL"));
				stmt.setTimestamp("DATA_FINAL", (Timestamp) parametros.get("DATA_FINAL"));

				ResultSet rset = null;
				rset = stmt.executeQuery();

				lsDadosRelatorio = new ArrayList<RelatorioDocumentosFiscaisBean>();

				BigDecimal totalValorContabil = BigDecimal.ZERO;
				BigDecimal totalValorBaseCalculo = BigDecimal.ZERO;
				BigDecimal totalValorAliquiotaICMS = BigDecimal.ZERO;
				BigDecimal totalValorICMS = BigDecimal.ZERO;
				BigDecimal totalValorIsentas = BigDecimal.ZERO;
				BigDecimal totalValorOutras = BigDecimal.ZERO;
				BigDecimal totalValorCancelado = BigDecimal.ZERO;

				while (rset.next()) {

					RelatorioDocumentosFiscaisBean bean = new RelatorioDocumentosFiscaisBean();
					bean.setNumAIDF((String) (rset.getObject("numAIDF") == null ? "" : rset.getObject("numAIDF")));
					bean.setFormInicial((String) rset.getObject("formInicial"));
					bean.setFormFinal((String) rset.getObject("formFinal"));
					bean.setSerie((String) rset.getObject("serie"));
					bean.setSubSerie((String) rset.getObject("subSerie"));
					bean.setValorContabil((BigDecimal) rset.getObject("valorContabil"));
					bean.setValorBaseCalculo((BigDecimal) rset.getObject("valorBaseCalculo"));
					bean.setValorAliquiotaICMS((BigDecimal) rset.getObject("valorAliquiotaICMS"));
					bean.setValorICMS((BigDecimal) rset.getObject("valorICMS"));
					bean.setOutras((BigDecimal) rset.getObject("outras"));
					bean.setValorCancelado((BigDecimal) rset.getObject("valorCancelado"));
					bean.setNomeEstado((String) rset.getObject("nomeEstado"));
					bean.setNomeOrigem((String) rset.getObject("nomeOrigem"));
					bean.setNomeDestino((String) rset.getObject("nomeDestino"));
					lsDadosRelatorio.add(bean);

				}

				if (lsDadosRelatorio.size() > 0) {

					for (RelatorioDocumentosFiscaisBean bean : lsDadosRelatorio) {

						bean.setValorContabil(bean.getValorContabil().setScale(2, RoundingMode.HALF_EVEN));
						bean.setValorBaseCalculo(bean.getValorBaseCalculo().setScale(2, RoundingMode.HALF_EVEN));
						bean.setValorAliquiotaICMS(bean.getValorAliquiotaICMS().setScale(2, RoundingMode.HALF_EVEN));
						bean.setValorICMS(bean.getValorICMS().setScale(2, RoundingMode.HALF_EVEN));
						bean.setOutras(bean.getOutras().setScale(2, RoundingMode.HALF_EVEN));
						bean.setValorCancelado(bean.getValorCancelado().setScale(2, RoundingMode.HALF_EVEN));

						totalValorContabil = totalValorContabil.add(bean.getValorContabil() != null ? bean.getValorContabil() : BigDecimal.ZERO);
						totalValorBaseCalculo = totalValorBaseCalculo.add(bean.getValorBaseCalculo() != null ? bean.getValorBaseCalculo() : BigDecimal.ZERO);
						totalValorAliquiotaICMS = totalValorAliquiotaICMS.add(bean.getValorAliquiotaICMS() != null ? bean.getValorAliquiotaICMS() : BigDecimal.ZERO);
						totalValorICMS = totalValorICMS.add(bean.getValorICMS() != null ? bean.getValorICMS() : BigDecimal.ZERO);
						totalValorOutras = totalValorOutras.add(bean.getOutras() != null ? bean.getOutras() : BigDecimal.ZERO);
						totalValorCancelado = totalValorCancelado.add(bean.getValorCancelado() != null ? bean.getValorCancelado() : BigDecimal.ZERO);
					}
					setLsDadosRelatorio(lsDadosRelatorio);

					parametros.put("TOTAL_VALOR_CONTABIL", totalValorContabil);
					parametros.put("TOTAL_VALOR_BASE_CALCULO", totalValorBaseCalculo);
					parametros.put("TOTAL_VALOR_ALIQUIOTA_ICMS", totalValorAliquiotaICMS);
					parametros.put("TOTAL_VALOR_ICMS", totalValorICMS);
					parametros.put("TOTAL_VALOR_ISENTAS", totalValorIsentas);
					parametros.put("TOTAL_VALOR_OUTRAS", totalValorOutras);
					parametros.put("TOTAL_VALOR_CANCELADO", totalValorCancelado);

				}
			}
		});

	}

	public void setLsDadosRelatorio(List<RelatorioDocumentosFiscaisBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	private String getSql(Integer empresaId, String agencia, Boolean somenteCancelado, String tipoLinha, Aidf aidf) {

		StringBuilder sql = new StringBuilder();

		sql.append("SELECT coalesce(tabela.numAidf, '') AS numAidf, ");
		sql.append("       TABELA.formInicial AS formInicial, ");
		sql.append("       TABELA.formFinal AS formFinal, ");
		sql.append("       TABELA.serie AS serie, ");
		sql.append("       TABELA.subSerie AS subSerie, ");
		sql.append("       TABELA.nomeEstado AS nomeEstado, ");
		sql.append("       TABELA.origem AS nomeOrigem, ");
		sql.append("       TABELA.destino AS nomeDestino, ");
		sql.append("       COALESCE(SUM(tabela.valorContabil),0) AS valorContabil, ");
		sql.append("       SUM(valorBaseCalculo) AS valorBaseCalculo, ");
		sql.append("       TABELA.valorAliquiotaICMS AS valorAliquiotaICMS, ");
		sql.append("       SUM(TABELA.valorICMS) AS valorICMS, ");
		sql.append("       COALESCE(SUM(TABELA.valorCancelado),0) AS valorCancelado,  ");
		sql.append("       SUM(TABELA.outras) AS outras ");
		sql.append("FROM ");
		sql.append("  (SELECT a.ACFISCAL AS numAidf, ");
		sql.append("          a.FORMINICIAL AS formInicial, ");
		sql.append("          a.FORMFINAL AS formFinal, ");
		sql.append("          a.SERIE AS serie, ");
		sql.append("          a.SUBSERIE AS subSerie, ");
		sql.append("          c.MOTIVOCANCELACION_ID AS motivoCancelacion, ");
		sql.append("          c.PRECIOBASE AS tarifa, ");
		sql.append("          ed.ICMS AS valorAliquiotaICMS, ");
		sql.append("          CASE  WHEN c.MOTIVOCANCELACION_ID IS NULL THEN c.PRECIOPAGADO    ELSE 0 END AS valorContabil, ");
		sql.append("          CASE  WHEN c.MOTIVOCANCELACION_ID IS NOT NULL THEN c.PRECIOPAGADO  ELSE 0 END AS valorCancelado, ");
		sql.append("          COALESCE(ei.PORCREDBASEICMS / 100,0) AS redBaseCalcIcms, ");
		sql.append("          CASE  WHEN c.MOTIVOCANCELACION_ID IS NULL THEN c.PRECIOPAGADO - (c.PRECIOPAGADO * COALESCE(ei.PORCREDBASEICMS / 100,0)) ELSE 0  END AS valorBaseCalculo, ");
		sql.append("          CASE  WHEN c.MOTIVOCANCELACION_ID IS NULL THEN ((c.PRECIOPAGADO - (c.PRECIOPAGADO * COALESCE(ei.PORCREDBASEICMS / 100,0))) * ed.ICMS) / 100ELSE 0  END AS valorICMS, ");
		sql.append("          CASE  WHEN c.MOTIVOCANCELACION_ID IS NULL THEN c.PRECIOPAGADO - (c.PRECIOPAGADO - (c.PRECIOPAGADO * COALESCE(ei.PORCREDBASEICMS / 100,0))) ELSE 0 END AS outras, ");
		sql.append("          eo.estado_id AS estadoId, eo.NOMBESTADO AS nomeEstado, po.DESCPARADA AS origem,  pd.DESCPARADA AS destino, ");
		sql.append("          CASE WHEN((COALESCE(eos.cveestado, eo.cveestado)) = (COALESCE(eds.cveestado, ed.cveestado))) THEN 0  ELSE 1 END AS isInterEstadual ");
		sql.append("   FROM caja c ");
		sql.append("   INNER JOIN aidf a ON a.aidf_id = c.aidf_id ");
		sql.append("   JOIN marca m ON c.marca_id = m.marca_id ");
		sql.append("   JOIN empresa e ON e.empresa_id = m.empresa_id ");
		sql.append("   JOIN parada po ON po.parada_id = c.origen_id ");
		sql.append("   JOIN ciudad co ON co.ciudad_id = po.ciudad_id ");
		sql.append("   JOIN estado eo ON eo.estado_id = co.estado_id ");
		sql.append("   JOIN parada pd ON pd.parada_id = c.destino_id ");
		sql.append("   JOIN ciudad cd ON cd.ciudad_id = pd.ciudad_id ");
		sql.append("   JOIN estado ed ON ed.estado_id = cd.estado_id ");
		sql.append("   LEFT JOIN alias_servico s ON s.origen_id = c.origen_id ");
		sql.append("   AND s.destino_id = c.destino_id ");
		sql.append("   AND (s.corrida_id = c.corrida_id ");
		sql.append("        OR s.corrida_id IS NULL) ");
		sql.append("   AND s.ruta_id = c.ruta_id ");
		sql.append("   LEFT JOIN parada pos ON pos.parada_id = s.aliasorigen_id ");
		sql.append("   LEFT JOIN ciudad cos ON cos.ciudad_id = pos.ciudad_id ");
		sql.append("   LEFT JOIN estado eos ON eos.estado_id = cos.estado_id ");
		sql.append("   LEFT JOIN parada pds ON pds.parada_id = s.aliasdestino_id ");
		sql.append("   LEFT JOIN ciudad cds ON cds.ciudad_id = pds.ciudad_id ");
		sql.append("   LEFT JOIN estado eds ON eds.estado_id = cds.estado_id ");
		sql.append("   INNER JOIN empresa_imposto ei ON ei.empresa_id = e.empresa_id ");
		sql.append("     WHERE c.feccreacion between :DATA_INICIAL and :DATA_FINAL ");
		sql.append("AND c.EMPRESACORRIDA_ID =" + empresaId + " ");
		sql.append(somenteCancelado == true ? " AND c.INDCANCELACION = 1" : " ");
		sql.append("AND a.AIDF_ID = " + aidf.getAidfId() + " ");
		sql.append(")TABELA ");
		sql.append(tipoLinha.equals("INTERMUNICIPAL") ? " where TABELA.isInterEstadual = 1" : tipoLinha.equals("INTERESTADUAL") ? " where TABELA.isInterEstadual = 0" : " ");
		sql.append("GROUP BY TABELA.numAidf,TABELA.formInicial, ");
		sql.append("         TABELA.formFinal,TABELA.serie,  TABELA.subSerie,TABELA.nomeEstado, ");
		sql.append("         TABELA.origem, tabela.destino, ");
		sql.append("         TABELA.valorAliquiotaICMS ");
		sql.append("ORDER BY TABELA.nomeEstado,TABELA.origem,  TABELA.destino");

		return sql.toString();
	}

}
