package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.zhtml.Filedownload;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;

import com.rjconsultores.ventaboletos.ArquivoRemessa;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FechamentoBoleto;
import com.rjconsultores.ventaboletos.layouts.CNABFactory;
import com.rjconsultores.ventaboletos.layouts.RemessaInterface;
import com.rjconsultores.ventaboletos.service.ArquivoRemessaCNAB;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderFechamentoBoleto;

@Controller("relatorioArquivoRemessaCNABController")
@Scope("prototype")
public class RelatorioArquivoRemessaCNABController extends MyGenericForwardComposer {
	
	
	private MyListbox boletoFechamentoList;
	
	private List<Empresa> lsEmpresas;
	private List<FechamentoBoleto> lsFechamentoBoletos;
	private Empresa empresa;
	
	private Combobox cmbEmpresa;
	private Datebox fecBoletoIni;
	private Datebox fecBoletoFin;
	
	@Autowired
	private ArquivoRemessaCNAB arquivoRemessaCNAB;
	
	@Override
    public void doAfterCompose(Component comp) throws Exception {
		
		this.lsFechamentoBoletos = this.arquivoRemessaCNAB.obtenerTodosParaRemessa(null, null, null);
		this.lsEmpresas = this.arquivoRemessaCNAB.getEmpresas();

        super.doAfterCompose(comp);
        
        for (Empresa e : this.lsEmpresas) {
			Comboitem comboItem = new Comboitem(e.getNombempresa());
			comboItem.setValue(e);
			comboItem.setParent(this.cmbEmpresa);
		}
        
        this.boletoFechamentoList.setItemRenderer(new RenderFechamentoBoleto());
        this.boletoFechamentoList.setData(this.lsFechamentoBoletos);
        
	}
	
	public void onClick$btnPesquisar(Event ev) throws Exception {
		
		Empresa empresa = (Empresa) cmbEmpresa.getSelectedItem().getValue();
		this.lsFechamentoBoletos = arquivoRemessaCNAB.obtenerTodosParaRemessa(empresa, this.fecBoletoIni.getValue(), this.fecBoletoFin.getValue());
		this.boletoFechamentoList.setData(this.lsFechamentoBoletos);
	}
	
	public void onClick$btnExecutarRemessa(Event ev) throws Exception {
		
		empresa = (Empresa) cmbEmpresa.getSelectedItem().getValue();
		
		if(empresa == null){
			
			
			return;
		}
		
		if(fecBoletoIni.getValue() == null || fecBoletoFin.getValue() == null){
			
			
			return;
		}
		
		ArquivoRemessa arquivoRemessa = arquivoRemessaCNAB.remessa(empresa, fecBoletoIni.getValue(), fecBoletoFin.getValue());
		
		RemessaInterface remessaInterface = CNABFactory.createRemessa(arquivoRemessa.getBancoLayout());
		
		File fileRemessa = new File(arquivoRemessa.getNomeArquivo());
		
		List<String> conteudoArquivoRemessa = remessaInterface.gerarRemessa(fileRemessa, arquivoRemessa);
		
		InputStream is = null;
        is = new FileInputStream(fileRemessa);
        Filedownload.save(is, "application/txt", arquivoRemessa.getNomeArquivo());
        
        arquivoRemessaCNAB.atualizaRemessa(empresa, arquivoRemessa);
		
	}
	
	public void onClick$btnImportarRetornoRemessa(Event ev) throws Exception {
		
		
	}
	
	public Empresa getEmpresa() {
		return empresa;
	}
	
	public void setEmpresa(Empresa empresa) {
		this.empresa = empresa;
	}
}
